@@ -31,16 +31,17 @@ import os
 import shutil
 import textwrap
 import tempfile
-import atexit
 
 from contextlib import contextmanager
 from uuid import uuid4
 
-from yaml import safe_load
+from yaml import safe_dump, safe_load
 
 from ansible_runner import run
 from ansible_runner import output
-from ansible_runner.utils import dump_artifact, Bunch
+from ansible_runner import cleanup
+from ansible_runner.utils import dump_artifact, Bunch, register_for_cleanup
+from ansible_runner.utils.capacity import get_cpu_count, get_mem_in_bytes, ensure_uuid
 from ansible_runner.runner import Runner
 from ansible_runner.exceptions import AnsibleRunnerException
 
@@ -407,31 +408,20 @@ DEFAULT_CLI_ARGS = {
             )
         ),
     ),
-    "execenv_cli_group": (
-        (
-            ('--container-runtime',),
-            dict(
-                dest='container_runtime',
-                default='podman',
-                help="OCI Compliant container runtime to use. Examples: podman, docker"
-            ),
-        ),
-        (
-            ('--keep-files',),
-            dict(
-                dest='keep_files',
-                action='store_true',
-                default=False,
-                help="Keep temporary files persistent on disk instead of cleaning them automatically. "
-                     "(Useful for debugging)"
-            ),
-        ),
-    ),
 }
 
 logger = logging.getLogger('ansible-runner')
 
 
+class AnsibleRunnerArgumentParser(argparse.ArgumentParser):
+    def error(self, message):
+        # If no sub command was provided, print common usage then exit
+        if 'required: command' in message.lower():
+            print_common_usage()
+
+        super(AnsibleRunnerArgumentParser, self).error(message)
+
+
 @contextmanager
 def role_manager(vargs):
     if vargs.get('role'):
@@ -558,11 +548,9 @@ def add_args_to_parser(parser, args):
     """
     Traverse a tuple of argments to add to a parser
 
-    :param parser: Instance of a parser, subparser, or argument group
-    :type sys_args: argparse.ArgumentParser
+    :param argparse.ArgumentParser parser: Instance of a parser, subparser, or argument group
 
-    :param args: Tuple of tuples, format ((arg1, arg2), {'kwarg1':'val1'},)
-    :type sys_args: tuple
+    :param tuple args: Tuple of tuples, format ((arg1, arg2), {'kwarg1':'val1'},)
 
     :returns: None
     """
@@ -576,14 +564,13 @@ def main(sys_args=None):
     When the ```ansible-runner``` command is executed, this function
     is the main entry point that is called and executed.
 
-    :param sys_args: List of arguments to be parsed by the parser
-    :type sys_args: list
+    :param list sys_args: List of arguments to be parsed by the parser
 
     :returns: an instance of SystemExit
     :rtype: SystemExit
     """
 
-    parser = argparse.ArgumentParser(
+    parser = AnsibleRunnerArgumentParser(
         prog='ansible-runner',
         description="Use 'ansible-runner' (with no arguments) to see basic usage"
     )
@@ -630,12 +617,40 @@ def main(sys_args=None):
         'worker',
         help="Execute work streamed from a controlling instance"
     )
+    worker_subcommands = worker_subparser.add_subparsers(
+        help="Sub-sub command to invoke",
+        dest='worker_subcommand',
+        description="ansible-runner worker [sub-sub-command]",
+    )
+    cleanup_command = worker_subcommands.add_parser(
+        'cleanup',
+        help="Cleanup private_data_dir patterns from prior jobs and supporting temporary folders.",
+    )
+    cleanup.add_cleanup_args(cleanup_command)
+
     worker_subparser.add_argument(
         "--private-data-dir",
         help="base directory containing the ansible-runner metadata "
              "(project, inventory, env, etc)",
     )
 
+    worker_subparser.add_argument(
+        "--worker-info",
+        dest="worker_info",
+        action="store_true",
+        help="show the execution node's Ansible Runner version along with its memory and CPU capacities"
+    )
+    worker_subparser.add_argument(
+        "--delete",
+        dest="delete_directory",
+        action="store_true",
+        default=False,
+        help=(
+            "Delete existing folder (and everything in it) in the location specified by --private-data-dir. "
+            "The directory will be re-populated when the streamed data is unpacked. "
+            "Using this will also assure that the directory is deleted when the job finishes."
+        )
+    )
     process_subparser = subparser.add_parser(
         'process',
         help="Receive the output of remote ansible-runner work and distribute the results"
@@ -747,25 +762,49 @@ def main(sys_args=None):
     add_args_to_parser(isalive_container_group, DEFAULT_CLI_ARGS['container_group'])
     add_args_to_parser(transmit_container_group, DEFAULT_CLI_ARGS['container_group'])
 
-    if len(sys.argv) == 1:
-        parser.print_usage()
-        print_common_usage()
-        parser.exit(status=0)
-
     args = parser.parse_args(sys_args)
 
     vargs = vars(args)
 
-    if vargs.get('command') in ('worker', 'process'):
+    if vargs.get('command') == 'worker':
+        if vargs.get('worker_subcommand') == 'cleanup':
+            cleanup.run_cleanup(vargs)
+            parser.exit(0)
+        if vargs.get('worker_info'):
+            cpu = get_cpu_count()
+            mem = get_mem_in_bytes()
+            errors = []
+            uuid = ensure_uuid()
+            if not isinstance(mem, int):
+                errors.append(mem)
+                mem = None
+            if "Could not find" in uuid:
+                errors.append(uuid)
+                uuid = None
+            info = {'errors': errors,
+                    'mem_in_bytes': mem,
+                    'cpu_count': cpu,
+                    'runner_version': VERSION,
+                    'uuid': uuid,
+                    }
+            print(safe_dump(info, default_flow_style=True))
+            parser.exit(0)
+
+        private_data_dir = vargs.get('private_data_dir')
+        delete_directory = vargs.get('delete_directory', False)
+        if private_data_dir and delete_directory:
+            shutil.rmtree(private_data_dir, ignore_errors=True)
+            register_for_cleanup(private_data_dir)
+        elif private_data_dir is None:
+            temp_private_dir = tempfile.mkdtemp()
+            vargs['private_data_dir'] = temp_private_dir
+            register_for_cleanup(temp_private_dir)
+
+    if vargs.get('command') == 'process':
+        # the process command is the final destination of artifacts, user expects private_data_dir to not be cleaned up
         if not vargs.get('private_data_dir'):
             temp_private_dir = tempfile.mkdtemp()
             vargs['private_data_dir'] = temp_private_dir
-            if vargs.get('keep_files', False):
-                print("ANSIBLE-RUNNER: keeping temporary data directory: {}".format(temp_private_dir))
-            else:
-                @atexit.register
-                def conditonally_clean_cli_execenv_tempdir():
-                    shutil.rmtree(temp_private_dir)
 
     if vargs.get('command') in ('start', 'run', 'transmit'):
         if vargs.get('hosts') and not (vargs.get('module') or vargs.get('role')):
@@ -854,6 +893,7 @@ def main(sys_args=None):
                                    resource_profiling_memory_poll_interval=vargs.get('resource_profiling_memory_poll_interval'),
                                    resource_profiling_pid_poll_interval=vargs.get('resource_profiling_pid_poll_interval'),
                                    resource_profiling_results_dir=vargs.get('resource_profiling_results_dir'),
+                                   cmdline=vargs.get('cmdline'),
                                    limit=vargs.get('limit'),
                                    streamer=streamer
                                    )
