@@ -14,18 +14,38 @@ import threading
 import pipes
 import uuid
 import codecs
+import atexit
+import signal
 
 from distutils.spawn import find_executable
+
 from ansible_runner.exceptions import ConfigurationError
 
 try:
-    from collections.abc import Iterable, Mapping
+    from collections.abc import Iterable, MutableMapping
 except ImportError:
-    from collections import Iterable, Mapping
+    from collections import Iterable, MutableMapping
 from io import StringIO
 from six import string_types, PY2, PY3, text_type, binary_type
 
 
+def cleanup_folder(folder):
+    """Deletes folder, returns True or False based on whether a change happened."""
+    try:
+        shutil.rmtree(folder)
+        return True
+    except (FileNotFoundError, NotADirectoryError):
+        return False
+
+
+def register_for_cleanup(folder):
+    '''
+    Provide the path to a folder to make sure it is deleted when execution finishes.
+    The folder need not exist at the time when this is called.
+    '''
+    atexit.register(cleanup_folder, folder)
+
+
 class Bunch(object):
 
     '''
@@ -53,7 +73,7 @@ def isplaybook(obj):
     Returns:
         boolean: True if the object is a list and False if it is not
     '''
-    return isinstance(obj, Iterable) and (not isinstance(obj, string_types) and not isinstance(obj, Mapping))
+    return isinstance(obj, Iterable) and (not isinstance(obj, string_types) and not isinstance(obj, MutableMapping))
 
 
 def isinventory(obj):
@@ -66,7 +86,7 @@ def isinventory(obj):
     Returns:
         boolean: True if the object is an inventory dict and False if it is not
     '''
-    return isinstance(obj, Mapping) or isinstance(obj, string_types)
+    return isinstance(obj, MutableMapping) or isinstance(obj, string_types)
 
 
 def check_isolation_executable_installed(isolation_executable):
@@ -183,15 +203,20 @@ def dump_artifacts(kwargs):
 
         kwargs['envvars']['ANSIBLE_ROLES_PATH'] = roles_path
 
-    obj = kwargs.get('playbook')
-    if obj and isplaybook(obj):
-        path = os.path.join(private_data_dir, 'project')
-        kwargs['playbook'] = dump_artifact(json.dumps(obj), path, 'main.json')
+    playbook = kwargs.get('playbook')
+    if playbook:
+        # Ensure the play is a list of dictionaries
+        if isinstance(playbook, MutableMapping):
+            playbook = [playbook]
+
+        if isplaybook(playbook):
+            path = os.path.join(private_data_dir, 'project')
+            kwargs['playbook'] = dump_artifact(json.dumps(playbook), path, 'main.json')
 
     obj = kwargs.get('inventory')
     if obj and isinventory(obj):
         path = os.path.join(private_data_dir, 'inventory')
-        if isinstance(obj, Mapping):
+        if isinstance(obj, MutableMapping):
             kwargs['inventory'] = dump_artifact(json.dumps(obj), path, 'hosts.json')
         elif isinstance(obj, string_types):
             if not os.path.exists(obj):
@@ -377,13 +402,17 @@ def ensure_str(s, encoding='utf-8', errors='strict'):
     """
     Copied from six==1.12
 
-    Coerce *s* to `str`.
+    Coerce *s* to ``str``.
+
     For Python 2:
-      - `unicode` -> encoded to `str`
-      - `str` -> `str`
+
+      - ``unicode`` -> encoded to ``str``
+      - ``str`` -> ``str``
+
     For Python 3:
-      - `str` -> `str`
-      - `bytes` -> decoded to `str`
+
+      - ``str`` -> ``str``
+      - ``bytes`` -> decoded to ``str``
     """
     if not isinstance(s, (text_type, binary_type)):
         raise TypeError("not expecting type '%s'" % type(s))
@@ -399,7 +428,10 @@ def sanitize_container_name(original_name):
     Docker and podman will only accept certain characters in container names
     This takes a given name from user-specified values and replaces the
     invalid characters so it can be used in docker/podman CLI commands
+
+    :param str original_name: Container name containing potentially invalid characters
     """
+
     return re.sub('[^a-zA-Z0-9_-]', '_', text_type(original_name))
 
 
@@ -412,6 +444,10 @@ def cli_mounts():
                     'src': '{}/.ssh/'.format(os.environ['HOME']),
                     'dest': '/home/runner/.ssh/'
                 },
+                {
+                    'src': '{}/.ssh/'.format(os.environ['HOME']),
+                    'dest': '/root/.ssh/'
+                },
                 {
                     'src': '/etc/ssh/ssh_known_hosts',
                     'dest': '/etc/ssh/ssh_known_hosts'
@@ -423,10 +459,10 @@ def cli_mounts():
 
 def santize_json_response(data):
     '''
-    Removes warning message from response message emitted by ansible
+    Removes warning message from response message emitted by Ansible
     command line utilities.
-    :param action: The string data to be santizied
-    :type action: str
+
+    :param str data: The string data to be sanitized
     '''
     start_re = re.compile("{(.|\n)*", re.MULTILINE)
     data = start_re.search(data).group().strip()
@@ -438,3 +474,20 @@ def get_executable_path(name):
     if exec_path is None:
         raise ConfigurationError(f"{name} command not found")
     return exec_path
+
+
+def signal_handler():
+    # Only the main thread is allowed to set a new signal handler
+    if threading.current_thread() is not threading.main_thread():
+        return None
+
+    signal_event = threading.Event()
+
+    # closure to set signal event
+    def _handler(number, frame):
+        signal_event.set()
+
+    signal.signal(signal.SIGTERM, _handler)
+    signal.signal(signal.SIGINT, _handler)
+
+    return signal_event.is_set
