@@ -4,12 +4,11 @@ from functools import partial
 from io import StringIO
 import os
 import re
-
 import six
+
 from pexpect import TIMEOUT, EOF
 
 import pytest
-from unittest.mock import (Mock, patch, PropertyMock)
 
 from ansible_runner.config.runner import RunnerConfig, ExecutionMode
 from ansible_runner.interface import init_runner
@@ -30,8 +29,9 @@ def load_file_side_effect(path, value=None, *args, **kwargs):
     raise ConfigurationError
 
 
-@patch('os.makedirs', return_value=True)
-def test_runner_config_init_defaults(mock_makedirs):
+def test_runner_config_init_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
     assert rc.private_data_dir == '/'
     assert rc.ident is not None
@@ -44,14 +44,16 @@ def test_runner_config_init_defaults(mock_makedirs):
     assert isinstance(rc.loader, ArtifactLoader)
 
 
-@patch('os.makedirs', return_value=True)
-def test_runner_config_with_artifact_dir(mock_makedirs):
+def test_runner_config_with_artifact_dir(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/', artifact_dir='/this-is-some-dir')
     assert rc.artifact_dir == os.path.join('/this-is-some-dir', rc.ident)
 
 
-@patch('os.makedirs', return_value=True)
-def test_runner_config_init_with_ident(mock_makedirs):
+def test_runner_config_init_with_ident(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/', ident='test')
     assert rc.private_data_dir == '/'
     assert rc.ident == 'test'
@@ -64,47 +66,53 @@ def test_runner_config_init_with_ident(mock_makedirs):
     assert isinstance(rc.loader, ArtifactLoader)
 
 
-@patch('os.makedirs', return_value=True)
-def test_runner_config_project_dir(mock_makedirs):
+def test_runner_config_project_dir(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/', project_dir='/another/path')
     assert rc.project_dir == '/another/path'
     rc = RunnerConfig('/')
     assert rc.project_dir == '/project'
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_environment_vars_only_strings(mock_makedirs):
+def test_prepare_environment_vars_only_strings(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig(private_data_dir="/", envvars=dict(D='D'))
 
     value = dict(A=1, B=True, C="foo")
     envvar_side_effect = partial(load_file_side_effect, 'env/envvars', value)
 
-    with patch.object(rc.loader, 'load_file', side_effect=envvar_side_effect):
-        rc.prepare_env()
-        assert 'A' in rc.env
-        assert isinstance(rc.env['A'], six.string_types)
-        assert 'B' in rc.env
-        assert isinstance(rc.env['B'], six.string_types)
-        assert 'C' in rc.env
-        assert isinstance(rc.env['C'], six.string_types)
-        assert 'D' in rc.env
-        assert rc.env['D'] == 'D'
+    mocker.patch.object(rc.loader, 'load_file', side_effect=envvar_side_effect)
+
+    rc.prepare_env()
+    assert 'A' in rc.env
+    assert isinstance(rc.env['A'], six.string_types)
+    assert 'B' in rc.env
+    assert isinstance(rc.env['B'], six.string_types)
+    assert 'C' in rc.env
+    assert isinstance(rc.env['C'], six.string_types)
+    assert 'D' in rc.env
+    assert rc.env['D'] == 'D'
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_ad_hoc_command(mock_makedirs):
+def test_prepare_env_ad_hoc_command(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig(private_data_dir="/")
 
     value = {'AD_HOC_COMMAND_ID': 'teststring'}
     envvar_side_effect = partial(load_file_side_effect, 'env/envvars', value)
 
-    with patch.object(rc.loader, 'load_file', side_effect=envvar_side_effect):
-        rc.prepare_env()
-        assert rc.cwd == '/'
+    mocker.patch.object(rc.loader, 'load_file', side_effect=envvar_side_effect)
+
+    rc.prepare_env()
+    assert rc.cwd == '/'
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_environment_pexpect_defaults(mock_makedirs):
+def test_prepare_environment_pexpect_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig(private_data_dir="/")
     rc.prepare_env()
 
@@ -115,93 +123,135 @@ def test_prepare_environment_pexpect_defaults(mock_makedirs):
     assert rc.expect_passwords[EOF] is None
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_passwords(mock_makedirs):
+def test_prepare_env_passwords(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig(private_data_dir='/')
 
     value = {'^SSH [pP]assword.*$': 'secret'}
     password_side_effect = partial(load_file_side_effect, 'env/passwords', value)
 
-    with patch.object(rc.loader, 'load_file', side_effect=password_side_effect):
-        rc.prepare_env()
-        rc.expect_passwords.pop(TIMEOUT)
-        rc.expect_passwords.pop(EOF)
-        assert len(rc.expect_passwords) == 1
-        assert isinstance(list(rc.expect_passwords.keys())[0], Pattern)
-        assert 'secret' in rc.expect_passwords.values()
+    mocker.patch.object(rc.loader, 'load_file', side_effect=password_side_effect)
+
+    rc.prepare_env()
+    rc.expect_passwords.pop(TIMEOUT)
+    rc.expect_passwords.pop(EOF)
+    assert len(rc.expect_passwords) == 1
+    assert isinstance(list(rc.expect_passwords.keys())[0], Pattern)
+    assert 'secret' in rc.expect_passwords.values()
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_extra_vars_defaults(mock_makedirs):
+def test_prepare_env_extra_vars_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
     rc.prepare_env()
     assert rc.extra_vars is None
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_settings_defaults(mock_makedirs):
+def test_prepare_env_settings_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
     rc.prepare_env()
     assert rc.settings == {}
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_settings(mock_makedirs):
+def test_prepare_env_settings(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
 
     value = {'test': 'string'}
     settings_side_effect = partial(load_file_side_effect, 'env/settings', value)
 
-    with patch.object(rc.loader, 'load_file', side_effect=settings_side_effect):
-        rc.prepare_env()
-        assert rc.settings == value
+    mocker.patch.object(rc.loader, 'load_file', side_effect=settings_side_effect)
+
+    rc.prepare_env()
+    assert rc.settings == value
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_sshkey_defaults(mock_makedirs):
+def test_prepare_env_sshkey_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
     rc.prepare_env()
     assert rc.ssh_key_data is None
 
 
-@patch('ansible_runner.config._base.open_fifo_write')
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_sshkey(mock_makedirs, open_fifo_write_mock):
+def test_prepare_env_sshkey(mocker):
+    mocker.patch('ansible_runner.config._base.open_fifo_write')
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig('/')
 
     value = '01234567890'
     sshkey_side_effect = partial(load_file_side_effect, 'env/ssh_key', value)
 
-    with patch.object(rc.loader, 'load_file', side_effect=sshkey_side_effect):
-        rc.prepare_env()
-        assert rc.ssh_key_data == value
+    mocker.patch.object(rc.loader, 'load_file', side_effect=sshkey_side_effect)
+
+    rc.prepare_env()
+    assert rc.ssh_key_data == value
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_defaults(mock_makedirs):
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc = RunnerConfig('/')
-        rc.prepare_env()
-        assert rc.idle_timeout is None
-        assert rc.job_timeout is None
-        assert rc.pexpect_timeout == 5
-        assert rc.cwd == '/project'
+def test_prepare_env_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc = RunnerConfig('/')
+    rc.prepare_env()
+    assert rc.idle_timeout is None
+    assert rc.job_timeout is None
+    assert rc.pexpect_timeout == 5
+    assert rc.cwd == '/project'
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_env_directory_isolation(mock_makedirs):
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc = RunnerConfig('/')
-        rc.directory_isolation_path = '/tmp/foo'
-        rc.prepare_env()
-        assert rc.cwd == '/tmp/foo'
+def test_prepare_env_directory_isolation(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc = RunnerConfig('/')
+    rc.directory_isolation_path = '/tmp/foo'
+    rc.prepare_env()
+    assert rc.cwd == '/tmp/foo'
 
 
-@patch('os.makedirs', return_value=True)
-@patch('os.path.exists', return_value=True)
-def test_prepare_inventory(path_exists, mock_makedirs):
+def test_prepare_env_directory_isolation_from_settings(mocker, project_fixtures):
+    '''
+    Test that sandboxing with directory isolation works correctly with `env/settings` values.
+    '''
+    # Mock away the things that would actually prepare the isolation directory.
+    mocker.patch('os.makedirs', return_value=True)
+    copy_tree = mocker.patch('distutils.dir_util.copy_tree')
+    mkdtemp = mocker.patch('tempfile.mkdtemp')
+    mkdtemp.return_value = '/tmp/runner/runner_di_XYZ'
+    mocker.patch('ansible_runner.config.runner.RunnerConfig.build_process_isolation_temp_dir')
+
+    # The `directory_isolation` test data sets up an `env/settings` file for us.
+    private_data_dir = project_fixtures / 'directory_isolation'
+    rc = RunnerConfig(private_data_dir=str(private_data_dir), playbook='main.yaml')
+
+    # This is where all the magic happens
+    rc.prepare()
+
+    assert rc.sandboxed
+    assert rc.process_isolation_executable == 'bwrap'
+    assert rc.project_dir == str(private_data_dir / 'project')
+    assert os.path.exists(rc.project_dir)
+
+    # `directory_isolation_path` should be used to create a new temp path underneath
+    assert rc.directory_isolation_path == '/tmp/runner/runner_di_XYZ'
+    mkdtemp.assert_called_once_with(prefix='runner_di_', dir='/tmp/runner')
+
+    # The project files should be copied to the isolation path.
+    copy_tree.assert_called_once_with(rc.project_dir, rc.directory_isolation_path, preserve_symlinks=True)
+
+
+def test_prepare_inventory(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+    path_exists = mocker.patch('os.path.exists', return_value=True)
+
     rc = RunnerConfig(private_data_dir='/')
     rc.prepare_inventory()
     assert rc.inventory == '/inventory'
@@ -217,12 +267,35 @@ def test_prepare_inventory(path_exists, mock_makedirs):
     assert rc.inventory is None
 
 
-@patch('os.makedirs', return_value=True)
-def test_generate_ansible_command(mock_makedirs):
+@pytest.mark.parametrize(
+    'extra_vars, expected',
+    (
+        ({'test': 'key'}, ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '{"test":"key"}', 'main.yaml']),
+        ('/tmp/extravars.yml', ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '@/tmp/extravars.yml', 'main.yaml']),
+        (None, ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', 'main.yaml']),
+    )
+)
+def test_generate_ansible_command_extra_vars(mocker, extra_vars, expected):
+    mocker.patch('os.makedirs', return_value=True)
+    mocker.patch('os.path.exists', return_value=True)
+
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml')
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare_inventory()
+    rc.inventory = '/inventory'
+    rc.prepare_inventory()
+
+    mocker.patch.object(rc.loader, 'isfile', side_effect=lambda x: True)
+
+    rc.extra_vars = extra_vars
+    cmd = rc.generate_ansible_command()
+    assert cmd == expected
+
+
+def test_generate_ansible_command(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+    mocker.patch('os.path.exists', return_value=True)
+
+    rc = RunnerConfig(private_data_dir='/', playbook='main.yaml')
+    rc.prepare_inventory()
     rc.extra_vars = None
 
     cmd = rc.generate_ansible_command()
@@ -231,16 +304,6 @@ def test_generate_ansible_command(mock_makedirs):
     rc.extra_vars = dict(test="key")
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '{"test":"key"}', 'main.yaml']
-
-    with patch.object(rc.loader, 'isfile', side_effect=lambda x: True):
-        cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '{"test":"key"}', 'main.yaml']
-        rc.extra_vars = '/tmp/extravars.yml'
-        cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '@/tmp/extravars.yml', 'main.yaml']
-        rc.extra_vars = None
-        cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', 'main.yaml']
     rc.extra_vars = None
 
     rc.inventory = "localhost,"
@@ -254,16 +317,17 @@ def test_generate_ansible_command(mock_makedirs):
     rc.inventory = []
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', 'main.yaml']
+
     rc.inventory = None
 
-    with patch('os.path.exists', return_value=False) as path_exists:
-        rc.prepare_inventory()
-        cmd = rc.generate_ansible_command()
+    mocker.patch('os.path.exists', return_value=False)
+    rc.prepare_inventory()
+    cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', 'main.yaml']
 
     rc.verbosity = 3
-    with patch('os.path.exists', return_value=True) as path_exists:
-        rc.prepare_inventory()
+    mocker.patch('os.path.exists', return_value=True)
+    rc.prepare_inventory()
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', '-i', '/inventory', '-vvv', 'main.yaml']
     rc.verbosity = None
@@ -290,23 +354,25 @@ def test_generate_ansible_command(mock_makedirs):
     assert cmd == ['ansible-playbook', '-i', '/inventory', '--forks', '5', 'main.yaml']
 
 
-@patch('os.makedirs', return_value=True)
-def test_generate_ansible_command_with_api_extravars(mock_makedirs):
+def test_generate_ansible_command_with_api_extravars(mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml', extravars={"foo": "bar"})
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare_inventory()
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc.prepare_inventory()
 
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '{"foo":"bar"}', 'main.yaml']
 
 
-@patch('os.makedirs', return_value=True)
-def test_generate_ansible_command_with_dict_extravars(mock_makedirs):
+def test_generate_ansible_command_with_dict_extravars(mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml', extravars={"foo": "test \n hello"})
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare_inventory()
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc.prepare_inventory()
 
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '{"foo":"test \\n hello"}', 'main.yaml']
@@ -316,22 +382,25 @@ def test_generate_ansible_command_with_dict_extravars(mock_makedirs):
     (u'--tags foo --skip-tags', ['--tags', 'foo', '--skip-tags']),
     (u'--limit "䉪ቒ칸ⱷ?噂폄蔆㪗輥"', ['--limit', '䉪ቒ칸ⱷ?噂폄蔆㪗輥']),
 ])
-@patch('os.makedirs', return_value=True)
-def test_generate_ansible_command_with_cmdline_args(mock_makedirs, cmdline, tokens):
+def test_generate_ansible_command_with_cmdline_args(cmdline, tokens, mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml')
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare_inventory()
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc.prepare_inventory()
     rc.extra_vars = {}
 
     cmdline_side_effect = partial(load_file_side_effect, 'env/cmdline', cmdline)
-    with patch.object(rc.loader, 'load_file', side_effect=cmdline_side_effect):
-        cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook'] + tokens + ['-i', '/inventory', 'main.yaml']
+    mocker.patch.object(rc.loader, 'load_file', side_effect=cmdline_side_effect)
+
+    cmd = rc.generate_ansible_command()
+    assert cmd == ['ansible-playbook'] + tokens + ['-i', '/inventory', 'main.yaml']
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_command_defaults(mock_makedirs):
+def test_prepare_command_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
 
     cmd_side_effect = partial(load_file_side_effect, 'args')
@@ -339,19 +408,21 @@ def test_prepare_command_defaults(mock_makedirs):
     def generate_side_effect():
         return StringIO(u'test "string with spaces"')
 
-    with patch.object(rc.loader, 'load_file', side_effect=cmd_side_effect):
-        with patch.object(rc, 'generate_ansible_command', side_effect=generate_side_effect):
-            rc.prepare_command()
-            rc.command == ['test', '"string with spaces"']
+    mocker.patch.object(rc.loader, 'load_file', side_effect=cmd_side_effect)
+    mocker.patch.object(rc, 'generate_ansible_command', side_effect=generate_side_effect)
+
+    rc.prepare_command()
+    rc.command == ['test', '"string with spaces"']
 
 
-@patch('os.makedirs', return_value=True)
-def test_prepare_with_defaults(mock_makedirs):
+def test_prepare_with_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
 
-    rc.prepare_inventory = Mock()
-    rc.prepare_env = Mock()
-    rc.prepare_command = Mock()
+    rc.prepare_inventory = mocker.Mock()
+    rc.prepare_env = mocker.Mock()
+    rc.prepare_command = mocker.Mock()
 
     rc.ssh_key_data = None
     rc.artifact_dir = '/'
@@ -363,15 +434,16 @@ def test_prepare_with_defaults(mock_makedirs):
     assert str(exc.value) == 'No executable for runner to run'
 
 
-@patch.dict('os.environ', {'PYTHONPATH': '/python_path_via_environ',
-                           'AWX_LIB_DIRECTORY': '/awx_lib_directory_via_environ'})
-@patch('os.makedirs', return_value=True)
-def test_prepare(mock_makedirs):
+def test_prepare(mocker):
+    mocker.patch.dict('os.environ', {
+        'PYTHONPATH': '/python_path_via_environ',
+        'AWX_LIB_DIRECTORY': '/awx_lib_directory_via_environ',
+    })
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
-
-    rc.prepare_inventory = Mock()
-    rc.prepare_command = Mock()
-
+    rc.prepare_inventory = mocker.Mock()
+    rc.prepare_command = mocker.Mock()
     rc.ssh_key_data = None
     rc.artifact_dir = '/'
     rc.env = {}
@@ -400,15 +472,15 @@ def test_prepare(mock_makedirs):
         "PYTHONPATH is the union of the explicit env['PYTHONPATH'] override and AWX_LIB_DIRECTORY"
 
 
-@patch('os.makedirs', return_value=True)
-@patch('ansible_runner.config._base.open_fifo_write')
-def test_prepare_with_ssh_key(open_fifo_write_mock, mock_makedirs):
+def test_prepare_with_ssh_key(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+    open_fifo_write_mock = mocker.patch('ansible_runner.config._base.open_fifo_write')
     rc = RunnerConfig('/')
 
-    rc.prepare_inventory = Mock()
-    rc.prepare_command = Mock()
+    rc.prepare_inventory = mocker.Mock()
+    rc.prepare_command = mocker.Mock()
 
-    rc.wrap_args_with_ssh_agent = Mock()
+    rc.wrap_args_with_ssh_agent = mocker.Mock()
 
     rc.ssh_key_data = None
     rc.artifact_dir = '/'
@@ -418,16 +490,17 @@ def test_prepare_with_ssh_key(open_fifo_write_mock, mock_makedirs):
     rc.ssh_key_data = '01234567890'
     rc.command = 'ansible-playbook'
 
-    with patch.dict('os.environ', {'AWX_LIB_DIRECTORY': '/'}):
-        rc.prepare()
+    mocker.patch.dict('os.environ', {'AWX_LIB_DIRECTORY': '/'})
+
+    rc.prepare()
 
     assert rc.ssh_key_path == '/ssh_key_data'
     assert rc.wrap_args_with_ssh_agent.called
     assert open_fifo_write_mock.called
 
 
-@patch('os.makedirs', return_value=True)
-def test_wrap_args_with_ssh_agent_defaults(mock_makedirs):
+def test_wrap_args_with_ssh_agent_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig('/')
     res = rc.wrap_args_with_ssh_agent(['ansible-playbook', 'main.yaml'], '/tmp/sshkey')
     assert res == [
@@ -437,8 +510,8 @@ def test_wrap_args_with_ssh_agent_defaults(mock_makedirs):
     ]
 
 
-@patch('os.makedirs', return_value=True)
-def test_wrap_args_with_ssh_agent_with_auth(mock_makedirs):
+def test_wrap_args_with_ssh_agent_with_auth(mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig('/')
     res = rc.wrap_args_with_ssh_agent(['ansible-playbook', 'main.yaml'], '/tmp/sshkey', '/tmp/sshauth')
     assert res == [
@@ -448,8 +521,8 @@ def test_wrap_args_with_ssh_agent_with_auth(mock_makedirs):
     ]
 
 
-@patch('os.makedirs', return_value=True)
-def test_wrap_args_with_ssh_agent_silent(mock_makedirs):
+def test_wrap_args_with_ssh_agent_silent(mocker):
+    mocker.patch('os.makedirs', return_value=True)
     rc = RunnerConfig('/')
     res = rc.wrap_args_with_ssh_agent(['ansible-playbook', 'main.yaml'], '/tmp/sshkey', silence_ssh_add=True)
     assert res == [
@@ -459,14 +532,14 @@ def test_wrap_args_with_ssh_agent_silent(mock_makedirs):
     ]
 
 
-@patch('ansible_runner.runner_config.RunnerConfig.prepare')
-@patch('ansible_runner.interface.sys')
-@patch('ansible_runner.utils.subprocess')
 @pytest.mark.parametrize('executable_present', [True, False])
-def test_process_isolation_executable_not_found(mock_subprocess, mock_sys, mock_prepare, executable_present):
+def test_process_isolation_executable_not_found(executable_present, mocker):
+    mocker.patch('ansible_runner.runner_config.RunnerConfig.prepare')
+    mock_sys = mocker.patch('ansible_runner.interface.sys')
+    mock_subprocess = mocker.patch('ansible_runner.utils.subprocess')
     # Mock subprocess.Popen indicates if
     # process isolation executable is present
-    mock_proc = Mock()
+    mock_proc = mocker.Mock()
     mock_proc.returncode = (0 if executable_present else 1)
     mock_subprocess.Popen.return_value = mock_proc
 
@@ -479,69 +552,99 @@ def test_process_isolation_executable_not_found(mock_subprocess, mock_sys, mock_
         assert mock_sys.exit.called
 
 
-@patch('os.makedirs', return_value=True)
-def test_bwrap_process_isolation_defaults(mock_makedirs):
+def test_bwrap_process_isolation_defaults(mocker):
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
     rc.artifact_dir = '/tmp/artifacts'
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
     rc.process_isolation = True
     rc.process_isolation_executable = 'bwrap'
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare()
+
+    path_exists = mocker.patch('os.path.exists')
+    path_exists.return_value = True
+
+    rc.prepare()
 
     assert rc.command == [
         'bwrap',
         '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
         '--bind', '/', '/',
         '--chdir', '/project',
         'ansible-playbook', '-i', '/inventory', 'main.yaml',
     ]
 
 
-@patch('os.makedirs', return_value=True)
-@patch('shutil.copytree', return_value=True)
-@patch('tempfile.mkdtemp', return_value="/tmp/dirisolation/foo")
-@patch('os.chmod', return_value=True)
-@patch('shutil.rmtree', return_value=True)
-def test_bwrap_process_isolation_and_directory_isolation(mock_makedirs, mock_copytree, mock_mkdtemp,
-                                                         mock_chmod, mock_rmtree):
-    def new_exists(path):
+def test_bwrap_process_isolation_and_directory_isolation(mocker, patch_private_data_dir, tmp_path):
+
+    def mock_exists(path):
         if path == "/project":
             return False
         return True
+
+    class MockArtifactLoader:
+        def __init__(self, base_path):
+            self.base_path = base_path
+
+        def load_file(self, path, objtype=None, encoding='utf-8'):
+            raise ConfigurationError
+
+        def isfile(self, path):
+            return False
+
+    mocker.patch('ansible_runner.config.runner.os.makedirs', return_value=True)
+    mocker.patch('ansible_runner.config.runner.os.chmod', return_value=True)
+    mocker.patch('ansible_runner.config.runner.os.path.exists', mock_exists)
+    mocker.patch('ansible_runner.config._base.ArtifactLoader', new=MockArtifactLoader)
+
+    artifact_path = tmp_path / 'artifacts'
+    artifact_path.mkdir()
+
     rc = RunnerConfig('/')
-    rc.artifact_dir = '/tmp/artifacts'
-    rc.directory_isolation_path = '/tmp/dirisolation'
+    rc.artifact_dir = tmp_path / 'artifacts'
+    rc.directory_isolation_path = tmp_path / 'dirisolation'
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
     rc.process_isolation = True
     rc.process_isolation_executable = 'bwrap'
-    with patch('os.path.exists', new=new_exists):
-        rc.prepare()
+
+    rc.prepare()
 
     assert rc.command == [
         'bwrap',
         '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
         '--bind', '/', '/',
         '--chdir', os.path.realpath(rc.directory_isolation_path),
         'ansible-playbook', '-i', '/inventory', 'main.yaml',
     ]
 
 
-@patch('os.path.isdir', return_value=False)
-@patch('os.path.exists', return_value=True)
-@patch('os.makedirs', return_value=True)
-def test_process_isolation_settings(mock_isdir, mock_exists, mock_makedirs):
+def test_process_isolation_settings(mocker, tmp_path):
+    mocker.patch('os.path.isdir', return_value=False)
+    mocker.patch('os.path.exists', return_value=True)
+    mocker.patch('os.makedirs', return_value=True)
+
     rc = RunnerConfig('/')
-    rc.artifact_dir = '/tmp/artifacts'
+    rc.artifact_dir = tmp_path.joinpath('artifacts').as_posix()
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
     rc.process_isolation = True
@@ -549,45 +652,54 @@ def test_process_isolation_settings(mock_isdir, mock_exists, mock_makedirs):
     rc.process_isolation_hide_paths = ['/home', '/var']
     rc.process_isolation_show_paths = ['/usr']
     rc.process_isolation_ro_paths = ['/venv']
-    rc.process_isolation_path = '/tmp'
+    rc.process_isolation_path = tmp_path.as_posix()
 
-    with patch('os.path.exists') as path_exists:
-        path_exists.return_value = True
-        rc.prepare()
+    mocker.patch('os.path.exists', return_value=True)
+
+    rc.prepare()
     print(rc.command)
-    assert rc.command[0:8] == [
+    expected = [
         'not_bwrap',
         '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
     ]
+    index = len(expected)
+    assert rc.command[0:index] == expected
 
     # hide /home
-    assert rc.command[8] == '--bind'
-    assert 'ansible_runner_pi' in rc.command[9]
-    assert rc.command[10] == os.path.realpath('/home')  # needed for Mac
+    assert rc.command[index] == '--bind'
+    assert 'ansible_runner_pi' in rc.command[index + 1]
+    assert rc.command[index + 2] == os.path.realpath('/home')  # needed for Mac
 
     # hide /var
-    assert rc.command[11] == '--bind'
-    assert 'ansible_runner_pi' in rc.command[12]
-    assert rc.command[13] == '/var' or rc.command[13] == '/private/var'
+    assert rc.command[index + 3] == '--bind'
+    assert 'ansible_runner_pi' in rc.command[index + 4]
+    assert rc.command[index + 5] in ('/var', '/private/var')
 
     # read-only bind
-    assert rc.command[14:17] == ['--ro-bind', '/venv', '/venv']
+    assert rc.command[index + 6:index + 9] == ['--ro-bind', '/venv', '/venv']
 
     # root bind
-    assert rc.command[17:20] == ['--bind', '/', '/']
+    assert rc.command[index + 9:index + 12] == ['--bind', '/', '/']
 
     # show /usr
-    assert rc.command[20:23] == ['--bind', '/usr', '/usr']
+    assert rc.command[index + 12:index + 15] == ['--bind', '/usr', '/usr']
 
     # chdir and ansible-playbook command
-    assert rc.command[23:] == ['--chdir', '/project', 'ansible-playbook', '-i', '/inventory', 'main.yaml']
+    assert rc.command[index + 15:] == ['--chdir', '/project', 'ansible-playbook', '-i', '/inventory', 'main.yaml']
 
 
-@patch('os.mkdir', return_value=True)
-def test_profiling_plugin_settings(mock_mkdir):
+def test_profiling_plugin_settings(mocker):
+    mocker.patch('os.mkdir', return_value=True)
+
     rc = RunnerConfig('/')
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
@@ -600,10 +712,11 @@ def test_profiling_plugin_settings(mock_mkdir):
         '--sticky',
         '-g',
         'cpuacct,memory,pids:ansible-runner/{}'.format(rc.ident),
-        'ansible-playbook'
+        'ansible-playbook',
+        'main.yaml',
     ]
-    for index, element in enumerate(expected_command_start):
-        assert rc.command[index] == element
+
+    assert expected_command_start == rc.command
     assert 'main.yaml' in rc.command
     assert rc.env['ANSIBLE_CALLBACK_WHITELIST'] == 'cgroup_perf_recap'
     assert rc.env['CGROUP_CONTROL_GROUP'] == 'ansible-runner/{}'.format(rc.ident)
@@ -617,8 +730,9 @@ def test_profiling_plugin_settings(mock_mkdir):
     assert rc.env['CGROUP_DISPLAY_RECAP'] == 'False'
 
 
-@patch('os.mkdir', return_value=True)
-def test_profiling_plugin_settings_with_custom_intervals(mock_mkdir):
+def test_profiling_plugin_settings_with_custom_intervals(mocker):
+    mocker.patch('os.mkdir', return_value=True)
+
     rc = RunnerConfig('/')
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
@@ -633,10 +747,11 @@ def test_profiling_plugin_settings_with_custom_intervals(mock_mkdir):
     assert rc.env['CGROUP_PID_POLL_INTERVAL'] == '1.5'
 
 
-@patch('os.path.isdir', return_value=True)
-@patch('os.path.exists', return_value=True)
-def test_container_volume_mounting_with_Z(mock_isdir, mock_exists, tmpdir):
-    rc = RunnerConfig(str(tmpdir))
+def test_container_volume_mounting_with_Z(mocker, tmp_path):
+    mocker.patch('os.path.isdir', return_value=True)
+    mocker.patch('os.path.exists', return_value=True)
+
+    rc = RunnerConfig(str(tmp_path))
     rc.container_volume_mounts = ['/tmp/project_path:/tmp/project_path:Z']
     rc.container_name = 'foo'
     rc.env = {}
@@ -651,29 +766,30 @@ def test_container_volume_mounting_with_Z(mock_isdir, mock_exists, tmpdir):
         raise Exception('Could not find expected mount, args: {}'.format(new_args))
 
 
-@pytest.mark.parametrize('container_runtime', ['docker', 'podman'])
-@patch('os.path.isdir', return_value=True)
-@patch('os.path.exists', return_value=True)
-def test_containerization_settings(mock_isdir, mock_exists, tmpdir, container_runtime):
-    with patch('ansible_runner.runner_config.RunnerConfig.containerized', new_callable=PropertyMock) as mock_containerized:
-        rc = RunnerConfig(tmpdir)
-        rc.ident = 'foo'
-        rc.playbook = 'main.yaml'
-        rc.command = 'ansible-playbook'
-        rc.process_isolation = True
-        rc.process_isolation_executable = container_runtime
-        rc.container_image = 'my_container'
-        rc.container_volume_mounts = ['/host1:/container1', '/host2:/container2']
-        mock_containerized.return_value = True
-        rc.prepare()
+@pytest.mark.test_all_runtimes
+def test_containerization_settings(tmp_path, runtime, mocker):
+    mocker.patch('os.path.isdir', return_value=True)
+    mocker.patch('os.path.exists', return_value=True)
+    mock_containerized = mocker.patch('ansible_runner.runner_config.RunnerConfig.containerized', new_callable=mocker.PropertyMock)
+    mock_containerized.return_value = True
+
+    rc = RunnerConfig(tmp_path)
+    rc.ident = 'foo'
+    rc.playbook = 'main.yaml'
+    rc.command = 'ansible-playbook'
+    rc.process_isolation = True
+    rc.process_isolation_executable = runtime
+    rc.container_image = 'my_container'
+    rc.container_volume_mounts = ['/host1:/container1', '/host2:/container2']
+    rc.prepare()
 
     extra_container_args = []
-    if container_runtime == 'podman':
+    if runtime == 'podman':
         extra_container_args = ['--quiet']
     else:
-        extra_container_args = ['--user={os.getuid()}']
+        extra_container_args = [f'--user={os.getuid()}']
 
-    expected_command_start = [container_runtime, 'run', '--rm', '--tty', '--interactive', '--workdir', '/runner/project'] + \
+    expected_command_start = [runtime, 'run', '--rm', '--tty', '--interactive', '--workdir', '/runner/project'] + \
         ['-v', '{}/:/runner/:Z'.format(rc.private_data_dir)] + \
         ['-v', '/host1/:/container1/', '-v', '/host2/:/container2/'] + \
         ['--env-file', '{}/env.list'.format(rc.artifact_dir)] + \
@@ -681,8 +797,4 @@ def test_containerization_settings(mock_isdir, mock_exists, tmpdir, container_ru
         ['--name', 'ansible_runner_foo'] + \
         ['my_container', 'ansible-playbook', '-i', '/runner/inventory/hosts', 'main.yaml']
 
-    for index, element in enumerate(expected_command_start):
-        if '--user' in element:
-            assert '--user=' in rc.command[index]
-        else:
-            assert rc.command[index] == element
+    assert expected_command_start == rc.command
